@@ -104,15 +104,23 @@ static void resize_image(GtkImage *image, __attribute__((unused)) gpointer user_
 {
         GdkPixbuf *pixbuf;
         GdkPixbuf *scaled_pixbuf;
+        int pixbuf_height;
+        int pixbuf_width;
+        int pixbuf_size;
         g_debug("zzz resize_image");
 
         pixbuf = gtk_image_get_pixbuf (image);
 
         if (pixbuf == NULL) return;
 
-        if (gdk_pixbuf_get_height(pixbuf) != (current_icon_size-2)) {
+        pixbuf_height = gdk_pixbuf_get_height(pixbuf);
+        if ( pixbuf_height !=0 && pixbuf_height != (current_icon_size-2)) {
+                pixbuf_width = gdk_pixbuf_get_width(pixbuf);
+                pixbuf_size = (int)((double)(current_icon_size-2) / pixbuf_height * pixbuf_width );
+                if (pixbuf_width <= 0 || (current_icon_size-2) <= 0 || pixbuf_size <= 0) return;
+
                 scaled_pixbuf = gdk_pixbuf_scale_simple( pixbuf,
-                        (int)((double)(current_icon_size-2) / gdk_pixbuf_get_height(pixbuf) * gdk_pixbuf_get_width(pixbuf)),
+                        pixbuf_size,
                         (current_icon_size-2), GDK_INTERP_HYPER
                         );
 
@@ -302,6 +310,42 @@ static gboolean entry_scrolled(GtkWidget *menuitem, GdkEventScroll *event,
         return FALSE;
 }
 
+static gboolean
+entry_pressed (GtkWidget *menuitem, GdkEvent *event, __attribute__((unused)) gpointer data)
+{
+       g_return_val_if_fail(GTK_IS_MENU_ITEM(menuitem), FALSE);
+
+       if (((GdkEventButton*)event)->button == 2) /* middle button */
+       {
+               gtk_widget_grab_focus(menuitem);
+
+               return TRUE;
+       }
+
+       return FALSE;
+}
+
+static gboolean
+entry_released (GtkWidget *menuitem, GdkEvent *event, __attribute__((unused)) gpointer data)
+{
+       g_return_val_if_fail(GTK_IS_MENU_ITEM(menuitem), FALSE);
+
+       if (((GdkEventButton*)event)->button == 2) /* middle button */
+       {
+               IndicatorObject *io = g_object_get_data (G_OBJECT (menuitem), MENU_DATA_INDICATOR_OBJECT);
+               IndicatorObjectEntry *entry = g_object_get_data (G_OBJECT (menuitem), MENU_DATA_INDICATOR_ENTRY);
+
+               g_return_val_if_fail(INDICATOR_IS_OBJECT(io), FALSE);
+
+               g_signal_emit_by_name (io, INDICATOR_OBJECT_SIGNAL_SECONDARY_ACTIVATE, entry,
+                       ((GdkEventButton*)event)->time);
+
+               return TRUE;
+       }
+
+       return FALSE;
+}
+
 static void accessible_desc_update_cb(GtkWidget *widget, gpointer userdata)
 {
         gpointer data = g_object_get_data(G_OBJECT(widget), MENU_DATA_INDICATOR_ENTRY);
@@ -356,12 +400,16 @@ static void entry_added(IndicatorObject *io, IndicatorObjectEntry *entry, GtkWid
 
         /* Allows indicators to receive mouse scroll event in GTK+3 */
         gtk_widget_add_events(GTK_WIDGET(menuitem), GDK_SCROLL_MASK);
+        gtk_widget_add_events(GTK_WIDGET(menuitem), GDK_BUTTON_PRESS_MASK);
+	gtk_widget_add_events(GTK_WIDGET(menuitem), GDK_BUTTON_RELEASE_MASK);
 
         g_object_set_data(G_OBJECT(menuitem), "indicator", io);
         g_object_set_data(G_OBJECT(menuitem), "box", box);
 
         g_signal_connect(G_OBJECT(menuitem), "activate", G_CALLBACK(entry_activated), entry);
         g_signal_connect(G_OBJECT(menuitem), "scroll-event", G_CALLBACK(entry_scrolled), entry);
+        g_signal_connect(G_OBJECT(menuitem), "button-press-event", G_CALLBACK(entry_pressed), entry);
+	g_signal_connect(G_OBJECT(menuitem), "button-release-event", G_CALLBACK(entry_released), entry);
 
         if (entry->image != NULL) {
                 g_debug("zzz have an image");
@@ -604,9 +652,9 @@ static void menu_show(IndicatorObject *io, IndicatorObjectEntry *entry,
         if (entry == NULL) {
                 /* Close any open menus instead of opening one */
                 GList *entries = indicator_object_get_entries(io);
-                GList *entry = NULL;
-                for (entry = entries; entry != NULL; entry = g_list_next(entry)) {
-                        IndicatorObjectEntry *entrydata = (IndicatorObjectEntry *)entry->data;
+                GList * iterator = NULL;
+		for (iterator = entries; iterator != NULL; iterator = g_list_next(iterator)) {
+			IndicatorObjectEntry * entrydata = (IndicatorObjectEntry *)iterator->data;
                         gtk_menu_popdown(entrydata->menu);
                 }
                 g_list_free(entries);
@@ -705,6 +753,10 @@ load_indicators_from_indicator_files (AppIndicatorApplet *applet, GtkWidget *men
         const gchar *name;
         GError *error = NULL;
 
+        if (!g_file_test(INDICATOR_SERVICE_DIR, (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR))) {
+                return; // directory not available
+        }
+
         dir = g_dir_open (INDICATOR_SERVICE_DIR, 0, &error);
 
         if (!dir) {
@@ -726,12 +778,12 @@ load_indicators_from_indicator_files (AppIndicatorApplet *applet, GtkWidget *men
                 if (!g_strcmp0(name, INDICATOR_SERVICE_APPMENU_NG)) {
                         continue;
                 }
-                if (!g_strcmp0(name, INDICATOR_SERVICE_ME_NG)) {
+                /*if (!g_strcmp0(name, INDICATOR_SERVICE_ME_NG)) {
                         continue;
-                }
-                if (!g_strcmp0(name,  INDICATOR_SERVICE_DATETIME_NG)) {
+                }*/
+                /*if (!g_strcmp0(name,  INDICATOR_SERVICE_DATETIME_NG)) {
                         continue;
-                }
+                }*/
                 if (indicator) {
                         load_indicator(applet, menubar, INDICATOR_OBJECT (indicator), name);
                         count++;
